
##############################################################################
# Chapter 3                                                                  #
# Sanchez, J.Time Series for Data Scientists, CUP 2023.                        #
# Code used for the Case Study, Section 3.11                                #
# Program chapter3case.R in Base R version                                    #
# Last modified: 1/28/2023                                                   #
##############################################################################

# This is not strictly a Base R program. Base R and ts() will not handle 
## data with multiple seasonalities, but we want to see R's capabilities 
## with other packages before immersing into the fpp3 package. 

## Uncomment the install.packages() lines if they are alreadyinstalled 
#   install.packages("xts")
#   install.packages("zoo")
#   install.packages("dplyr")
#   install.packages("lubridate")
#   install.packages("dygraphs")
  library(xts)
  library(zoo)
  library(dplyr)
  library(lubridate)
  library(dygraphs)


# if the dataset is in the same directory as this program, read data thus:
  AEP <- read.csv("AEP_hourly.csv",header=T)
   head(AEP)
  vars=c("Datetime", "AEP_MW")
  dup=AEP[duplicated(AEP[,vars[1]]),vars] #check for duplicates
  dup   # daylight savings time duplicated time, so time 3 AM missing some years.

   dim(AEP)

    AEP[82490:82500,] # view one of the duplicates
    
## The date you see in the first line 
## is not the oldest observation. 
### The data is not sorted properly
  
     head(AEP,100)  # you can see that problem, scroll the Datetime
## The date you see in the last observation
## is not the most recent observation. 
## The data is not sorted properly. 
   tail(AEP,100)
   str(AEP)  # we notice that date is a character variable. A problem. 

   # there are alternatives, for example... 
#date = as.POSIXct(AEP$Datetime , format = "%Y-%m-%d %H:%M:%S ")

## but lets use lubridate to convert the 
## time stamp from a character variable to 
## a date variable. 
  
   date=as_datetime(AEP$Datetime)
   length(date)
   head(date)
   tail(date)

## notice how the creation of the date
## still maintains the wrong oldest 
## observation as the first line 

   sum(is.na(date))  # check again for missing values 
   ## In appearance there are no missing values, but as indicated 
   ## earlier we are missing some 3 AM observations when daylight savings. 
   ## they are just reported as duplictes of the previous hour time. 


############## Chunk of code useful to learn about the poor ordering of the data
## This commented out approach is useful to detect if something 
## is missing in the data. This is how we discovered that for 
## 14 days there isn't a 3:00 AM time (daylight savings)
#date = as.POSIXct(AEP$Datetime , format = "%Y-%m-%d %H:%M:%S ") #This works 
## notice that this date will contain some missing values, because 
# in the original date time there are some hour 3 missing. 
## I do not use this because it creates missing values. 
## Do the following to identify the rows that will give us missing 
### values 
### x=which(is.na(date)==T)
# x
#   AEP[x,] # tells us which values are missing... looks like they are there, but 
# they should be 3 AM instead of 2 AM. That is why these commented out pieces of 
## code are useful to detect something that otherwise would not be detected. 


#### Create an xts object for a ts plot. This is a more flexible time series 
## data class than ts() 
##This xts object is correct. 
    mw=AEP$AEP_MW
    length(mw)

    data.xts=xts(mw,order.by=date, frequency="hours")
    head(data.xts)

## Notice now how the corrects oldest 
## time stamp is in the top of the 
## data. The data is now properly 
## ordered in chronological order. 
    head(data.xts,40)

    tail(data.xts)
   class(data.xts)
    str(data.xts)
   


###  change title.
# Figure 3.12
#png("electricityts1.png", units="in", width=8.5, height=8, res=1200)
  par(
    mfrow=c(1,1),
    font.axis=2, 
    mar=c(5,5,5,5),
    font.main=2,
    font.lab=2
     )

 
    dygraph(data.xts,main="Energy Consumption with multiple seasonalities",
        ylab="Energy Consumption (MW") %>% dyRangeSelector
    dev.off()

### Figure 3.13.
### Plot again and move the range selector around to 
### observe the multiple seasonalities. 

 par(
  mfrow=c(1,1),
  font.axis=2, 
  mar=c(5,5,5,5),
  font.main=2,
  font.lab=2
   )

 
   dygraph(data.xts,main="Energy Consumption with multiple seasonalities.\n (2018 window)",
        ylab="Energy Consumption (MW") %>% dyRangeSelector
dev.off()




#### Further study of the seasonality at different 
## levels of granurality. 
## extract features of the time series

#### extract the hour, week and month 
   hour=hour(date); head(hour)
   year=year(date); head(year)
   week=week(date); head(week)
   day=day(date);head(day)
   length(hour)
   weekday=wday(date, label=TRUE, abbr=TRUE); head(weekday); length(weekday)
    month=factor(month.abb[month(date)],levels=month.abb); head(month)
   length(month)

### create data frame with features
  data=data.frame(year, month, day, week, weekday,  hour,   mw)
  newdata <- data[order(year,month,day,week,weekday, hour),]
  head(newdata)
  tail(newdata)
  
##inspecting seasonality by month, hour and weekday. 
 
  #Figure 3.14  
#png("electricitybox.png", units="in", width=8.5, height=8, res=1200)
  # added one more 
  par(
    mfrow=c(4,1),
    font.axis=2, 
    mar=c(5,5,5,5),
    font.main=2,
    font.lab=2
    )


   boxplot(mw~month,lwd=1.5,cex=1.5, 
         main="(a) Higher consumption in Winter and Summer months than other months",
          ylab="Energy consumption (MW)")
   boxplot(mw~hour,lwd=1.5,cex=1.5, 
        main="(b) Lower consumption during the night than during the day",
        ylab="Energy consumption (MW)" )
   boxplot(mw~weekday, lwd=1.5,cex=1.5, 
        main="(c) Lower consumption on Saturday and Sunday than other days of the week",
        ylab="Energy consumption (MW)")
  boxplot(mw~week, lwd=1.5,cex=1.5, 
        main="(c) Lower consumption on Saturday and Sunday than other days of the week",
        ylab="Energy consumption (MW)")
  dev.off()

   ##################### ACFs ##################

#### For the acf, we extract 
## the data that has been properly 
##sorted, the xts data set. 
  energy=coredata(data.xts)
  head(energy)
  energy=diff(energy)

# png("acfenergy.png", units="in", width=8.5, height=8, res=1200)
  par(
     mfrow=c(3,1),
     font.axis=2, 
     mar=c(5,5,5,5),
     font.main=2,
     font.lab=2
     )
## First day of the series is October 10, 
#time interval hour, cycle daily, frequency 24
# start in 284th day of the year, at hour 1. 
    test.day=ts(energy, start=c(284,1), frequency=24)
   acf(test.day, main="(a) The 1 lag means 1 cycle, \n in this picture 24 hours",
    lwd=1.5,cex=1.5) 
#time interval hour, cycle weekly, frequency 168 (24*7)
#Start at week 41 of the year at hour 145
    test.week=ts(energy, start=c(41,145), frequency=168)
    acf(test.week,lag=400,main="(b) The 1 lag means 1 cycle, \n in this picture 168 hours (a week)",
    lwd=1.5,cex=1.5)
#time interval hour, cycle yearly, frequency 8766 (24*365.25)
#start at year 2004, hour 6817
     test.year=ts(energy, start=c(2004,6817), frequency=8766)
    acf(test.year,lag=10000, main="(c) The 1 lag means 1 cycle, \n in this picture 8766 hours (a year)",
    lwd=1.5,cex=1.5)
   dev.off()

